﻿using System;
using System.IO;
using JJS.CrystalCMS.DBBackup.DASP.CustomerAPI;

namespace JJS.CrystalCMS.DBBackup
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                Log(string.Format("The backup process was started by the host {0}", Request.UserHostAddress));
                DeleteExistingBackup();

                CustomerApi wsAPI = new CustomerApi();

                string apiKey = (string)Application["apiKey"];
                //string apiKey = (string)Application["apiSandBoxKey"];
                string databaseName = (string)Application["databaseName"];

                if (wsAPI.VerifyKey(apiKey))
                {
                    Log(string.Format("The API key {0} was verified successfully", apiKey));
                    wsAPI.Sql2008CreateBackup(apiKey, databaseName);
                    Log(string.Format("The database {0} was successfully backed up", databaseName));
                }
                else
                    throw new Exception(string.Format("'{0}' is not a valid api key", apiKey));
            }
            catch (Exception ex)
            {
                string exMessage = string.Format("An error occurred : {0}", ex.Message);
                Log(exMessage);
                throw new Exception(exMessage);
            }
            finally
            {
                Log("Backup process completed");
                System.Diagnostics.Trace.Flush();
            }
        }

        private void DeleteExistingBackup()
        {
            //Only a single db.bak is allowed at DASP. We need to delete the existing backup if it exists prior to new backup.
            //Databases are always backed up to the _database folder at the root of the DASP account

            //This one will work if this app is installed at the root of the DASP account
            if (!DeleteExistingBackup(Server.MapPath("./_database")))
            {
                //This one will be used when this app is not installed at the root
                //Note - an assumption is made that this app is installed in a sub folder below the root so we're just getting the parent folder
                DirectoryInfo dInfo = new DirectoryInfo(Server.MapPath("."));
                string parentFolder = dInfo.Parent.FullName;
                DeleteExistingBackup(parentFolder + "\\_database");
            }
        }

        private bool DeleteExistingBackup(string backupPath)
        {           
            if (Directory.Exists(backupPath))
            {
                string[] files = Directory.GetFiles(backupPath, "*.bak");
                foreach (string fileName in files)
                {
                    File.Delete(fileName);
                    Log(string.Format("The old backup {0} was deleted", fileName));
                    return true;
                }
            }
            return false;
        }

        private void Log(string logString)
        {
            string strToLog = string.Format("{0} : {1}", DateTime.UtcNow, logString);
            Trace.Write(strToLog);
            System.Diagnostics.Trace.WriteLine(strToLog);
        }
    }
}
